////////////////////////////////////////////////////////////////////
//
//  FGODBCRecord.m
//  
//
//  Created by Francesco Germinara on Mon Jun 07 2004.
// 
//  Copyright (c) 2004 SofTech di F.Germinara - www.germinara.it. All rights reserved.
//
//  For information please contact me at info@germinara.it 
////////////////////////////////////////////////////////////////////
//  History:
//
//  Description:
//
////////////////////////////////////////////////////////////////////


#import "FGODBCRecord.h"
#import "FGODBCFieldInfo.h"


@implementation FGODBCRecord

-(id)init{
 self=[super init];
 FGODBCFieldsInfo=[[NSMutableDictionary alloc]init];	
 return self;
}


//Create Object: Default Constructor
-(id)initWithDataBase:(FGODBCConnection *) dbName forSQLTable:(NSString*) strName{
 self=[super init];
 nFields=0;
 if(dbName==nil){
  NSException* aException = [NSException exceptionWithName:@"FGODBCRecord" reason:@"FGODBCRecord->:DataBase non valido" userInfo:(id)self];
  [aException raise];	
 }
 
 database=dbName;	

 if(strName==nil){
	 NSException* aException = [NSException exceptionWithName:@"FGODBCRecord" reason:@"FGODBCRecord->:Tabella non valida" userInfo:(id)self];
	 [aException raise];	
 }
 
 strTableName=[[NSString alloc]initWithString:strName];
  
 FGODBCFieldsInfo=[[NSMutableDictionary alloc]init];
 
 //Alloco Handle di Istruzioni
 int sts=[database OpenCursor:&hstmt];
 if(sts!=0 || hstmt==0x00){
  NSException* aException = [NSException exceptionWithName:@"FGODBCRecord" reason:@"FGODBCRecord->:hstmt non valido!" userInfo:(id)self];
  [aException raise];	
 }
 
 nFields=0;
 return self;
}

//Return Table Name
-(NSString *)TableName{
 return strTableName;
} 


//Create Object copying data from another object
-(id)initWithRecordData:(FGODBCRecord *) record{
  self=[super init];
 
  database=record->database;

  strTableName=[[NSString alloc]initWithString:[record TableName]];
  
  nFields=record->nFields; 
  hstmt=record->hstmt;
  bIsEOF=record->bIsEOF; 
  bIsBOF=record->bIsBOF;
  
  
  FGODBCFieldsInfo=[[NSMutableDictionary alloc]init];

	  FGODBCFieldInfo *aFieldInfo;
	  NSString *strKey;
	  NSArray *arr=[record->FGODBCFieldsInfo allKeys];
	  
	  int nIndex=0;
	  
	  for(nIndex=0; nIndex<[arr count];nIndex++){
		  strKey=[arr objectAtIndex:nIndex];
		  aFieldInfo=[record FieldInfo:strKey];
		  FGODBCFieldInfo *tmpFieldInfo=[[FGODBCFieldInfo alloc]init];
		  
		  [tmpFieldInfo SetValues:[[NSString alloc]initWithString:[aFieldInfo name]] forType:[aFieldInfo Type] forPrecision:[aFieldInfo Precision] forScale:[aFieldInfo Scale] forNullability:[aFieldInfo Nullability] forCol:[aFieldInfo ColNum]];
		  [tmpFieldInfo SetValue:[[NSString alloc]initWithString:[aFieldInfo GetValue]]];
		  
		  [FGODBCFieldsInfo setObject:tmpFieldInfo forKey:strKey];
		  [tmpFieldInfo release];
	  }

  /*	  
  NSEnumerator *aEnum=[record->FGODBCFieldsInfo objectEnumerator];
   FGODBCFieldInfo *tmpFieldInfo;
   while((tmpFieldInfo = [aEnum nextObject])!=nil){
   }
 */
 
   	
  return(self);
}

//Close database handle and free resource, call before release
-(void)CloseCursor{
 [database CloseCursor:&hstmt]; //Close Communications Handle
}

//Standard destructor
-(void)dealloc{
 [FGODBCFieldsInfo removeAllObjects];//Libera anche gli oggetti allocati
 [FGODBCFieldsInfo release];
 [strTableName release];
 [super dealloc];
}

//Load all fields Informations and set the nFields value
-(void)FillFieldInfo{
	
 if(nFields!=0)
  return; //Already Done	 
	
 if(database==nil){
  NSException* aException = [NSException exceptionWithName:@"FGODBCRecord" reason:@"FGODBCRecord->:FillFieldInfo:DataBase non valido" userInfo:(id)self];
  [aException raise];	
 }
	
 [database FillColName:FGODBCFieldsInfo  HandleStatMent:&hstmt];
 nFields= [FGODBCFieldsInfo count];

if([strTableName isEqualToString:@"Commesse"]) //Modifica del 27/7/2005
 nFields--; //Last Field is a BLOB

}

//Return number of fields
-(int)GetNumFields{
 return nFields;
}

//return a Field Info Object
-(FGODBCFieldInfo *)FieldInfo:(NSString*)strName{
 FGODBCFieldInfo *aObjField=[FGODBCFieldsInfo objectForKey:strName];
 return(aObjField);
}

//return a Field Info Object
-(FGODBCFieldInfo *)FieldInfoByCol:(int)Col{
	FGODBCFieldInfo *aFieldInfo;
	NSString *strKey;
	NSArray *arr=[FGODBCFieldsInfo allKeys];
	
	int nIndex=0;
	
	for(nIndex=0; nIndex<[arr count];nIndex++){
		strKey=[arr objectAtIndex:nIndex];
		aFieldInfo=[self FieldInfo:strKey];
		if([aFieldInfo ColNum]==Col)
			return(aFieldInfo);
	}
	
	return(nil);
}


//Return Field Name of col (0 based)
-(NSString *)FieldName:(int)col{
 FGODBCFieldInfo *aFieldInfo;
 NSString *strKey;
 NSArray *arr=[FGODBCFieldsInfo allKeys];
 
 int nIndex=0;
 
 for(nIndex=0; nIndex<[arr count];nIndex++){
  strKey=[arr objectAtIndex:nIndex];
  aFieldInfo=[self FieldInfo:strKey];
  if([aFieldInfo ColNum]==col)
   return([aFieldInfo name]);
 }
 
 return(nil);
}

//Return Field Length of col (0 based)
-(int)FieldLegth:(int)col{
	
	FGODBCFieldInfo *aFieldInfo;
	NSString *strKey;
	NSArray *arr=[FGODBCFieldsInfo allKeys];
	
	int nIndex=0;
	
	for(nIndex=0; nIndex<[arr count];nIndex++){
		strKey=[arr objectAtIndex:nIndex];
		aFieldInfo=[self FieldInfo:strKey];
		if([aFieldInfo ColNum]==col)
			return([aFieldInfo length]);
	}
	return(0);
}

//Count record Items
-(long int)Count:(NSString *) strFilter{
 long int nRecords=0; //Number of Records
 int sts=0;
	
 NSMutableString *strQuery;
 strQuery=[[NSMutableString alloc] init];
 [strQuery appendString:@"select count(*) from "];
 [strQuery appendString:strTableName];
 [strQuery appendString:@" "];
 
 if([strFilter length]!=0)
  [strQuery appendString:strFilter];	 
 
 sts=[database ExecuteSQL:strQuery  HandleStatMent:&hstmt];
 
 if(sts==0){//Ok,
  sts=[database FindFirstRecord:&hstmt];
  if(sts==TRUE){
   nRecords=[[database GetFieldData:0  HandleStatMent:&hstmt] intValue]; 
  }
 }
	
 [database StopQuery:&hstmt];
 [strQuery release];
 return nRecords;
}

//Load First Row of Data
-(int)LoadDati:(NSString *) strFilter{
	int sts=0;
	int nIndex=0;

	NSMutableString *strQuery;
	strQuery=[[NSMutableString alloc] init];
	[strQuery appendString:@"select * from "];
	[strQuery appendString:strTableName];
	[strQuery appendString:@" "];
	
	if([strFilter length]!=0)
		[strQuery appendString:strFilter];	 
		
	sts=[database ExecuteSQL:strQuery  HandleStatMent:&hstmt];
	
	if(sts==0){//Ok,
			   //Carico i dati nei vari campi
		[self FillFieldInfo];
	
		sts=[database FindFirstRecord:&hstmt];
		if(sts==TRUE){
		 //Carico i dati nei vari campi
		 //[self FillFieldInfo];
			
		 for(nIndex=0; nIndex<nFields;nIndex++){
          FGODBCFieldInfo *aFieldInfo=[self FieldInfoByCol:nIndex];
          if([aFieldInfo Type] != -4){ //Blob Modifica del 27/7/2005
 		   NSString *tmpStr=[database GetFieldData:nIndex  HandleStatMent:&hstmt];
		   [aFieldInfo SetValue:tmpStr];
          }
		 }
		}
	}
    [database StopQuery:&hstmt];
	[strQuery release];
    return sts;	
}

-(void)DebugShowAllFields{
	NSMutableString *strMsg;
	strMsg=[[NSMutableString alloc] init];
	int nIndex=0;
	
	[strMsg appendFormat:@"Numero di Campi: %d\n",nFields];
	
	for(nIndex=0; nIndex<nFields;nIndex++){
	 FGODBCFieldInfo *aFieldInfo=[self FieldInfoByCol:nIndex];
      if([aFieldInfo Type] != -4){ //Blob Modifica del 27/7/2005
		[strMsg appendString:@"Nome: "];
		[strMsg appendString:[aFieldInfo name]];
		[strMsg appendString:@" Valore: ["];
		[strMsg appendString:[aFieldInfo GetValue]];
		[strMsg appendString:@"] Len: "];
		[strMsg appendFormat:@"%d",[aFieldInfo length]];
		[strMsg appendString:@"\n"];

     }
    }
	
	NSRunAlertPanel(@"Informazioni sul Record:",strMsg,@"OK",nil,nil);

	[strMsg release];	
}

//Open DataBase Cursor
-(int)OpenQuery:(NSString *) strFilter{
	int sts=0;
	int nIndex=0;

	NSMutableString *strQuery;
	strQuery=[[NSMutableString alloc] init];
	[strQuery appendString:@"select * from "];
	[strQuery appendString:strTableName];
	[strQuery appendString:@" "];
	
	bIsEOF=TRUE; 
	bIsBOF=TRUE;
	
	
	if([strFilter length]!=0)
		[strQuery appendString:strFilter];	 
		
	sts=[database ExecuteSQL:strQuery  HandleStatMent:&hstmt];
	
	if(sts==0){//Ok,
		sts=[database FindFirstRecord:&hstmt];
		if(sts==TRUE){
			bIsEOF=FALSE; 
			//Carico i dati nei vari campi
			[self FillFieldInfo];
			
			for(nIndex=0; nIndex<nFields;nIndex++){
				NSString *tmpStr=[database GetFieldData:nIndex  HandleStatMent:&hstmt];
				FGODBCFieldInfo *aFieldInfo=[self FieldInfoByCol:nIndex];
				[aFieldInfo SetValue:tmpStr];
			}
		}
	}
	[strQuery release];
    return sts;	
}

//Close DataBase Cursor
-(void)CloseQuery{
    [database StopQuery:&hstmt];
	bIsEOF=TRUE; 
	bIsBOF=TRUE;
}

-(int)MoveNext{
 int sts=0;	
 int nIndex=0;
 sts=[database MoveNextRecord:&hstmt];

	if(sts==TRUE){
		bIsEOF=FALSE; 
		//Carico i dati nei vari campi
		[self FillFieldInfo];
		
		for(nIndex=0; nIndex<nFields;nIndex++){
			NSString *tmpStr=[database GetFieldData:nIndex  HandleStatMent:&hstmt];
			FGODBCFieldInfo *aFieldInfo=[self FieldInfoByCol:nIndex];
			[aFieldInfo SetValue:tmpStr];
		}
	}else
	 bIsEOF=TRUE; 
 
	return sts;
}

//Try to Insert database with info fields contents
-(int)Insert{
 
 NSMutableString *strSqlInsert;
 int nIndex=0;
 int sts=0;
	
 strSqlInsert=[[NSMutableString alloc]init];
 [strSqlInsert appendFormat:@"INSERT INTO %@ (",strTableName];
 
 for(nIndex=0; nIndex<nFields-1; nIndex++){
  [strSqlInsert appendFormat:@"%@,",[[self FieldInfoByCol:nIndex] name]];
 }
 [strSqlInsert appendFormat:@"%@) VALUES (",[[self FieldInfoByCol:nIndex] name]];
	 
 for(nIndex=0; nIndex<nFields-1; nIndex++){
  [strSqlInsert appendFormat:@"'%@',",[[self FieldInfoByCol:nIndex] GetValueForUpdateSQL]];
 }
 [strSqlInsert appendFormat:@"'%@') ",[[self FieldInfoByCol:nIndex] GetValueForUpdateSQL]];
 
 sts=[database ExecuteSQL:strSqlInsert  HandleStatMent:&hstmt];

 [strSqlInsert release];  	
 return sts;
}


//[strInsQuery appendFormat:@"INSERT INTO fgusrcode (usrcode,nome,pw) VALUES ('MAK','Makky','MPW')"]; //Ovviamente se esiste mi da errore... 
//if([m_DB ExecuteSQL:strInsQuery]==-1){

// [strInsQuery appendFormat:@"UPDATE fgusrcode SET nome='UPDATE' WHERE usrcode='MAK' "]; //Update.
// [m_DB ExecuteSQL:strInsQuery];
//}

//Try to Update database with info fields contents
-(int)Update:(NSString *) strFilter{
	
	NSMutableString *strSqlUpdate;
	int nIndex=0;
	int sts=0;
	
	strSqlUpdate=[[NSMutableString alloc]init];
	[strSqlUpdate appendFormat:@"UPDATE %@ SET ",strTableName];
	
	for(nIndex=0; nIndex<nFields-1; nIndex++){
		[strSqlUpdate appendFormat:@"%@='",[[self FieldInfoByCol:nIndex] name]];
		[strSqlUpdate appendFormat:@"%@', ",[[self FieldInfoByCol:nIndex] GetValueForUpdateSQL]];
	}
	[strSqlUpdate appendFormat:@"%@='",[[self FieldInfoByCol:nIndex] name]];
	[strSqlUpdate appendFormat:@"%@' ",[[self FieldInfoByCol:nIndex] GetValueForUpdateSQL]];

	[strSqlUpdate appendString:strFilter];
	
	sts=[database ExecuteSQL:strSqlUpdate  HandleStatMent:&hstmt];
	
	[strSqlUpdate release];  

	return sts;
}

//Try to Delete record specified
-(int)Delete:(NSString *) strFilter{
	
	NSMutableString *strSqlDelete;
	int sts=0;
	
	strSqlDelete=[[NSMutableString alloc]init];
	[strSqlDelete appendFormat:@"DELETE FROM %@ ",strTableName];	
	[strSqlDelete appendString:strFilter];
	
	sts=[database ExecuteSQL:strSqlDelete  HandleStatMent:&hstmt];
	
	[strSqlDelete release];  	
	return sts;
}

//Add a field Info Manually
-(void)AddFieldInfoName:(NSString*) colName numCol:(short)colNum typeCol:(short) colType precisionCol:(UDWORD) colPrecision scaleCol:(short) colScale nullableCol:(short)colNullable strValore:(NSString*) strValueCol{
	NSString *strTxtNomeCampo;
	strTxtNomeCampo=[[NSString alloc]initWithString:colName];

	FGODBCFieldInfo *myInfoField;
	myInfoField=[[FGODBCFieldInfo alloc]init];
	
	[myInfoField SetValues:strTxtNomeCampo forType:colType forPrecision:colPrecision forScale:colScale forNullability:colNullable forCol:colNum];
	
	[myInfoField SetValue:strValueCol];
	
    [FGODBCFieldsInfo setObject:myInfoField forKey:(id)strTxtNomeCampo];
	
    [strTxtNomeCampo release];
	[myInfoField release];

	nFields= [FGODBCFieldsInfo count];
}

//Save BLOB record
-(int)SaveImage:(NSString *) strFilter fieldName:(NSString *)imageField  withImage:(unsigned char*) rowDataImage ofSize:(long int)fileSize{
 
	NSMutableString *strSqlUpdateBLOB;
	int sts=0;
	
	strSqlUpdateBLOB=[[NSMutableString alloc]init];
	[strSqlUpdateBLOB appendFormat:@"UPDATE %@ SET %@ = ? ",strTableName,imageField];
	[strSqlUpdateBLOB appendString:strFilter];

	sts=[database PutImage:strSqlUpdateBLOB  HandleStatMent:&hstmt ofSize:fileSize image:rowDataImage];
	
	[strSqlUpdateBLOB release];  

	return sts;
}

//Load BLOB record
- (unsigned char *) LoadImage:(NSString *) strFilter fieldName:(NSString *)imageField withSize:(long int *)bytes{
    unsigned char *result;
	NSMutableString *strSqlSelectBLOB;
	int sts=0;
	
	strSqlSelectBLOB=[[NSMutableString alloc]init];
	[strSqlSelectBLOB appendFormat:@"SELECT %@ FROM %@ ",imageField,strTableName];
	[strSqlSelectBLOB appendString:strFilter];

	result=[database GetImage:strSqlSelectBLOB  HandleStatMent:&hstmt withSize:bytes];
	
	[strSqlSelectBLOB release];  

	return result;
}


@end
